import React, {useContext, useEffect, useState} from "react";
import DOMPurify from "dompurify";
import SvgArrowDown from "../../icons/ArrowDown";
import SvgMessageRead from "../../icons/MessageRead";
import SvgMsgForwarded from "../../icons/MsgForwarded";
import SvgMsgStarred from "../../icons/MsgStarred";
import {Message, MessageReaction} from "../../utils/messages-data";
import { MessageMenu } from "../MessageMenu/MessageMenu";
import SvgMessageDelivered from "../../icons/MessageDelivered";
import {RobinChatContext} from "../../context/RobinChatContext";
import {DocumentMsg} from "../DocumentMsg/DocumentMsg";
import {ImageMsg} from "../ImageMsg/ImageMsg";
import {MsgReactionCount} from "../MsgReactionCount/MsgReactionCount";
import {LinkPreviewMsg} from "../LinkPreviewMsg/LinkPreviewMsg";
import {EMAIL_REGEX, WEBSITE_REGEX} from "../../utils/constants";
import {AudioMsg} from "../AudioMsg/AudioMsg";
import { State } from "../../context/state";
import { MsgReaction } from "../MsgReaction/MsgReaction";


interface Props {
  previousMsg: Message | undefined,
  msg: Message,
  allMessages: Array<Message>,
  isGroup: boolean | undefined
}

export const MessageBubble: React.FC<Props> = ({ msg, allMessages,  isGroup, previousMsg }) => {
  const [menuVisible, setMenuVisible] = useState<boolean>(false)
  const [reactions, setReactions] = useState<Array<string>>([])
  const [htmlMessage, setHtmlMessage] = useState()
  const {state, setState} = useContext(RobinChatContext)
  const messageRepliedTo = allMessages?.find(msgParam => msgParam._id === msg.reply_to)
  const messageSentByPreviousMessenger = previousMsg?.sender_token === msg?.sender_token
  const htmlMessagePlusLinks = updateTextWithLinks(msg)

  useEffect(() => {
    // Sort Message Reactions
    setReactions(sortMessageReactions(msg))
    setHtmlMessage(htmlMessagePlusLinks.html)
  }, [])

  useEffect(() => {
    if (!state?.dialogActive) setMenuVisible(false)
  }, [state?.dialogActive])

  useEffect(() => {
    const stateCopy: State = {...state}
    stateCopy.dialogActive = menuVisible
    if (setState) setState(stateCopy)
  }, [menuVisible])

  return (
    <div className={`msg__bubble_ctn ${state?.userToken === msg?.sender_token ? 'receiver' : 'sender'}`}>
      <div className={`msg__bubble ${state?.userToken === msg?.sender_token ? 'receiver' : 'sender'}`}>
        {/* FORWARD TEXT INDICATION */}
        {msg.is_forwarded && <div className="msg__bubble_forwarded icon__text">
          <SvgMsgForwarded/>
          <span className="text">Forwarded</span>
        </div>}

        {/* MESSAGE SENDER NAME */}
        {(state?.userToken !== msg?.sender_token && isGroup && !messageSentByPreviousMessenger) && <div className="msg__bubble_name">
          {msg?.sender_name || msg?.content?.sender_name}
        </div>}

        {/* MESSAGE RESPONDING CONTENT TEXT */}
        {msg.is_reply && <div className="msg__bubble_content msg__reply">
          { messageRepliedTo?.content?.msg || messageRepliedTo?.msg }
        </div>}

        {/* MESSAGE CONTENT TEXT */}
        {/* Hide message text content if it only contains "undefined" and it is a media file */}
        {!(msg?.content?.is_attachment && msg?.content.msg === 'undefined') && <div className="msg__bubble_content" dangerouslySetInnerHTML={{__html: htmlMessage}}>
        </div>}

        {/* MESSAGE CONTENT DOCUMENT */}
        <DocumentMsg message={msg} />

        {/* MESSAGE CONTENT IMAGE */}
        <ImageMsg message={msg} />

        {/* MESSAGE CONTENT AUDIO */}
        <AudioMsg message={msg} />

        {/* MESSAGE CONTENT IMAGES */}

        {/* MESSAGE LINK PREVIEW */}
        {msg?.link_meta_data?.map((metaData) => <LinkPreviewMsg metaData={metaData}/>)}

        {/* MESSAGE TIMESTAMP AND READ INDICATOR */}
        <div className="msg__time__indicator">
          {false && <div className="msg__star">
            <SvgMsgStarred/>
          </div>}
          <div className="msg__time">
            00:21
          </div>
          {state?.userToken === msg?.sender_token &&
          <div className="msg__indicator">
              {msg?.is_read && <SvgMessageRead/>}
              {!msg?.is_read && <SvgMessageDelivered/>}
          </div>}
        </div>

        {/* MORE MESSAGE OPTIONS */}
        <button className="icon__btn more__btn" onClick={() => setMenuVisible(true)}>
          <SvgArrowDown />
        </button>
        <MessageMenu
          setMenuVisible={setMenuVisible}
          animClass={menuVisible ? 'revealed' : ''}/>

        {/* REACTIONS */}
        <MsgReaction msg={msg} />

        {/* REACTIONS MADE */}
        {msg?.reactions && <MsgReactionCount reactions={reactions}/>}
      </div>
    </div>
  )
}

/**
 * Idea is to store message reactions in a comma seperated string
 * in the format " [reaction-string],[reaction-string-count]"
 * Value is stored in a set to avoid duplication and for performance reasons
 */
const sortMessageReactions = (msg: Message): Array<string> => {
  const tempReactions = new Set<string>()
  if (msg?.reactions) {
    for (const reaction of msg.reactions) {
      let reactionCount = 0
      for (const reactionInner of msg.reactions) {
        if (reactionInner.reaction === reaction.reaction) reactionCount++
      }
      tempReactions.add(`${reaction.reaction},${reactionCount}`)
    }
  }
  return Array.from(tempReactions)
}

const updateTextWithLinks = (msg: Message) => {
  let messageContent = msg?.content?.msg || msg?.msg
  const links = []

  // Website link matching
  const siteLinkMatches = messageContent?.match(WEBSITE_REGEX)
  if (siteLinkMatches) {
    for (const match of siteLinkMatches) {
      links.push(`https://${match}`)
      messageContent = messageContent?.replace(match, `<a class="robin_link" target="_blank" href="https://${match}">${match}</a>`)
    }
  }

  // Email link matching
  const emailLinkMatches = messageContent?.match(EMAIL_REGEX)
  if (emailLinkMatches) {
    for (const match of emailLinkMatches) {
      messageContent = messageContent?.replace(match, `<a class="robin_link" href="mailto:${match}">${match}</a>`)
    }
  }

  // Purify HTML
  messageContent = messageContent?.replace('https://', '')
    ?.replace('http://', '')
    ?.replace('https//', '')
    ?.replace('http//', '')
  const cleanMessageHTML = DOMPurify.sanitize(messageContent, {
    USE_PROFILES: { html: true },
  });
  return { html: cleanMessageHTML, links: links }
}
