import React, { Dispatch, SetStateAction, useContext, useEffect, useState } from "react";
import { ClipLoader } from "react-spinners";
import { RobinChatContext } from "../../context/RobinChatContext";
import { Checkbox } from "../../icons/Checkbox";
import SvgMultiply from "../../icons/Multiply";
import SvgSearch from "../../icons/Search";
import { ALPHABETS, CREATE_GROUP_HOME, CREATE_GROUP_PARTICIPANTS, SIDEBAR_STATE_ACTIVE } from "../../utils/constants";
import { User } from "../../utils/users-data";
import { usersByInitial } from "../NewChat/NewChat";
import { ParticipantsbyInitial } from "../ParticipantsbyInitial/ParticipantsbyInitial";


interface Props {
  groupName: string,
  groupImage: File | undefined,
  createGroupSection: string,
  setCreateGroupSection: Dispatch<SetStateAction<string>>,
  setSidebarPartVisible: Dispatch<SetStateAction<string>>
}

export const SelectGroupParticipants: React.FC<Props> = ({ groupName, groupImage, createGroupSection, setCreateGroupSection, setSidebarPartVisible }) => {
  const {state} = useContext(RobinChatContext)
  const users = state?.users || []
  const robin = state?.robin
  const [isAllSelected, setIsAllSelected] = useState<boolean>(false)
  const [isSearching, setIsSearching] = useState<boolean>(false)
  const [isLoading, setIsLoading] = useState<boolean>(false)
  const [selectedParticipants, setSelectedParticipants] = useState<Array<string>>([])
  const [searchedUsers, setSearchedUsers] = useState<Array<User>>(users.slice())

  // Participants Setter
  const addSelectedParticipant = (participantId: string) => {
    const selectedParticipantsCopy = [...selectedParticipants]

    if (!selectedParticipants.includes(participantId)) {
      selectedParticipantsCopy.push(participantId)
    } else {
      const idIndex = selectedParticipants.indexOf(participantId)
      selectedParticipantsCopy.splice(idIndex, 1)
    }
    setSelectedParticipants(selectedParticipantsCopy)
  }

  // Participants Setter
  const selectAllParticipants = () => {
    if (isAllSelected) {
      setSelectedParticipants([])
    } else {
      const selectedParticipantsCopy: Array<string> = []
      if (!isSearching) {
        users.forEach((user) => selectedParticipantsCopy.push(user._id))
      } else {
        searchedUsers.forEach((user) => selectedParticipantsCopy.push(user._id))
      }
      setSelectedParticipants(selectedParticipantsCopy)
    }
  }

  /**
   * Method to search all users that match a specified query
   * @param query string
   */
  const searchUsers = (query: string) => {
    if (query) {
      setIsSearching(true)
      setSearchedUsers(users.filter(user => user.fullname.toLowerCase().includes(query.toLowerCase())))
    } else {
      setIsSearching(false)
      setSearchedUsers(users.slice())
    }
  }

  const createRobinGroup = async () => {
    if (state?.userToken) {
      setIsLoading(true)
      // [participants] is an array that refines the [selectedParticipants] 
      // array from just an Array of string _ids
      const participants = selectedParticipants.map((userId) => {
        const user = users.find(user => user._id === userId)
        return {
          user_token: user?.user_token,
          meta_data: {
            profile_image: user?.profile_image,
            display_name: user?.fullname
          }
        }
      })
      const conversation = await robin?.createGroupConversation(groupName, { user_token: state.userToken, meta_data: { display_name: state?.username } }, participants)
      if (groupImage) await robin?.uploadGroupIcon(conversation.data._id, groupImage)
      setIsLoading(false)
      setSidebarPartVisible(SIDEBAR_STATE_ACTIVE)
    } else {
      throw new Error('userToken or File: groupImage is not defined')
    }
  }

  useEffect (() => {
    // Check If All Participants have been Selected. 
    if (selectedParticipants.length >= (isSearching ? searchedUsers.length : users.length)) {
      setIsAllSelected(true)
    } else {
      setIsAllSelected(false)
    }
  })

  return (
    <div className={`sec__sidebar participants__sidebar ${createGroupSection === CREATE_GROUP_PARTICIPANTS ? 'revealed' : ''}`}>
      <div className="sec__sidebar_header with__done">
        <div className="icon__text">
          <button className="icon__btn" onClick={() => setCreateGroupSection(CREATE_GROUP_HOME)}>
            <SvgMultiply />
          </button>
          <div className="text">Select Group Participants</div>
        </div>
        {!isLoading && <button className="icon__btn done__btn" disabled={selectedParticipants.length === 0}  onClick={() => createRobinGroup()}>
          Done
        </button>}
        <ClipLoader
          color="#15AE73"
          cssOverride={{borderWidth: '3px'}}
          loading={isLoading}
          size={28}
        />
      </div>

      {/* SEARCH PARTICIPANTS */}
      <div className="sidebar__row_2">
        <div className="form__input">
          <input type="text" className='icon__input' placeholder='Search people...' onChange={e => searchUsers(e.target.value)} />
          <div className="form__icon">
            <SvgSearch />
          </div>
        </div>
      </div>
      <button className="sidebar__archive_btn icon__btn spaced_icon__text select_all__btn" onClick={() => selectAllParticipants()}>
        <span className="text">Select All</span>
        <Checkbox value={isAllSelected} />
      </button>

      {/* PARTICIPANTS LIST */}
      {!isSearching && <div className="robin__archived_chats chat__list">
        { ALPHABETS?.map(letter => 
          <ParticipantsbyInitial 
            key={`users-by-${letter}`} 
            letter={letter} 
            users={usersByInitial(users, letter)}
            selectedParticipants={selectedParticipants}
            addSelectedParticipant={addSelectedParticipant} />) }
      </div>}

      {/* SEARCH PARTICIPANTS LIST */}
      {isSearching && <div className="robin__archived_chats chat__list">
        { ALPHABETS?.map(letter => 
          <ParticipantsbyInitial 
            key={`users-by-${letter}`} 
            letter={letter} 
            users={usersByInitial(searchedUsers, letter)}
            selectedParticipants={selectedParticipants}
            addSelectedParticipant={addSelectedParticipant} />) }
      </div>}
    </div>
  )
}